import md5
from urllib import quote, unquote

from DateTime import DateTime
from AccessControl.SecurityInfo import ClassSecurityInfo
from AccessControl.Permissions import view
from OFS.Folder import Folder
from App.class_init import default__class_init__ as InitializeClass
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Products.PageTemplates.ZopePageTemplate import ZopePageTemplate

from Products.PluggableAuthService.utils import classImplements
from Products.PluggableAuthService.plugins.BasePlugin import BasePlugin
from Products.PluggableAuthService.interfaces.plugins import IRolesPlugin
from Products.PluggableAuthService.interfaces.plugins import IExtractionPlugin
from Products.PluggableAuthService.interfaces.plugins import IAuthenticationPlugin


manage_addAutoRoleForm = PageTemplateFile(
    'www/autoRoleAdd', globals(), __name__='manage_addAutoRoleForm')


def addAutoRole( dispatcher
               , id
               , title=None
               , ip_roles=()
               , REQUEST=None
               ):
    """ Add an AutoRole plugin to a Pluggable Auth Service. """
    sp = AutoRole(id, title, ip_roles)
    dispatcher._setObject(sp.getId(), sp)

    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect( '%s/manage_workspace'
                                      '?manage_tabs_message='
                                      'AutoRole+added.'
                                    % dispatcher.absolute_url() )


class AutoRole(Folder, BasePlugin):
    """ Multi-plugin for assigning auto roles from IP. """

    meta_type = 'AutoRole'
    security = ClassSecurityInfo()

    _properties = ( { 'id'    : 'title'
                    , 'label' : 'Title'
                    , 'type'  : 'string'
                    , 'mode'  : 'w'
                    }
                  , { 'id'    : 'ip_roles'
                    , 'label' : 'IP filter and roles'
                    , 'type'  : 'lines'
                    , 'mode'  : 'w'
                    }
                  )

    manage_options = ( BasePlugin.manage_options[:1]
                     + Folder.manage_options[:1]
                     + Folder.manage_options[2:]
                     )

    def __init__(self, id, title=None, ip_roles=()):
        self._setId(id)
        self.title = title
        self.ip_roles = ip_roles

    def subnetmask(self, bits):
        "Convert.. for instance, 24 to '255.255.255.0'"
        ones = bits
        zeros = 32 - bits
        mask = ("1" * ones) + ("0" * zeros)
        mask = int(mask, 2)
        mask = self.base10_to_ip(mask)
        return mask

    def ip_to_base10(self, ip):
        "Given an IP address, convert to base10 integer"
        octets = ip.split('.')

        total = 0
        total += int(octets[3])
        total += int(octets[2]) * 256
        total += int(octets[1]) * 65536
        total += int(octets[0]) * 16777216

        return total


    def base10_to_ip(self, num):
        "Given an integer, convert to base2 dotted quad"
        oct1 = num / 16777216
        num = num % 16777216

        oct2 = num / 65536
        num = num % 65536

        oct3 = num / 256
        num = num % 256
    
        oct4 = num

        return str(oct1) + "." + str(oct2) + "." + str(oct3) + "." + str(oct4)

    def ip_in_range(self, ip, subnet):
        ip = self.ip_to_base10(ip)
        subnet, bits = subnet.split('/')
        ones = int(bits)
        zeros = 32 - ones
        mask = ("1" * ones) + ("0"* zeros)
        mask = int(mask, 2)
        size = pow(2, zeros)
        minimum = self.ip_to_base10(subnet) & mask
        maximum = minimum + size
        return minimum < ip < maximum

    #
    # IRolesPlugin
    #
    security.declarePrivate('getRolesForPrincipal')
    def getRolesForPrincipal(self, principal, request=None):
        """ Assign roles based on 'request'. """
        if request is None:
            request = getattr(self, 'REQUEST', {})
        ip = request.get('HTTP_X_FORWARDED_FOR') or request.get('REMOTE_ADDR', None)
        if ip is None:
            return []

        result = {}
        for line in self.ip_roles:
            subnet, roles = line.split(':')
            if self.ip_in_range(ip, subnet):
                [result.setdefault(role) for role in roles.split(',')]
        return result.keys()

    #
    # IExtractionPlugin
    #
    security.declarePrivate('extractCredentials')
    def extractCredentials(self, request):
        # Avoid creating anon user if this is a regular user
        # We actually have to poke request ourselves to avoid users from
        # root becoming anonymous...
        if getattr(request, '_auth', None):
            return {}

        # get client IP
        ip = request.get('HTTP_X_FORWARDED_FOR') or request.get('REMOTE_ADDR', None)
        if not ip:
            return {}

        for line in self.ip_roles:
            subnet, roles = line.split(':')
            if self.ip_in_range(ip, subnet):
                return {'AutoRole':True}

        return {}

    #
    # IAuthenticationPlugin
    #
    security.declarePrivate('authenticateCredentials')
    def authenticateCredentials(self, credentials):
        if credentials.has_key('login'):
            return None
        autorole = credentials.get('AutoRole', None)
        if not autorole:
            return None
        return ('Anonymous User', 'Anonymous User')


classImplements( AutoRole
               , IRolesPlugin
               , IExtractionPlugin
               , IAuthenticationPlugin
               )

InitializeClass(AutoRole)
