#
# AutoRoleTestCase AutoRole
#

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.PluggableAuthService.tests.conformance \
     import IRolesPlugin_conformance

from Products.PluggableAuthService.tests.test_PluggableAuthService \
     import FauxRequest, FauxResponse, FauxObject, FauxRoot, FauxContainer
from Products.PluggableAuthService.plugins.tests.test_CookieAuthHelper \
     import FauxSettableRequest, FauxCookieResponse


from Testing import ZopeTestCase
from Products.AutoRole.tests import AutoRoleTestCase


class TestAutoRole( AutoRoleTestCase.AutoRoleTestCase
                  , IRolesPlugin_conformance) :

    def _getTargetClass( self ):

        from Products.AutoRole.plugins.AutoRole \
            import AutoRole

        return AutoRole

    def _makeOne( self, id='test', *args, **kw ):

        return self._getTargetClass()( id=id, *args, **kw )

    def _makeTree( self ):

        rc = FauxObject( 'rc' )
        root = FauxRoot( 'root' ).__of__( rc )
        folder = FauxContainer( 'folder' ).__of__( root )
        object = FauxObject( 'object' ).__of__( folder )

        return rc, root, folder, object

    def afterSetUp(self):
        pass

    def test_getRolesForPrincipal( self ):
        helper = self._makeOne()
        response = FauxCookieResponse()
        request = FauxRequest(RESPONSE=response)
        # mask 8 = '255.0.0.0'
        # mask 32 = '255.255.255.255'
        # mask 24 = '255.255.255.0'

        # REMOTE_ADDR and HTTP_X_FORWARDED_FOR?
        # We assume that we're always behind a proxy that sets HTTP_X_FORWARDED_FOR correctly
        helper.ip_roles = ['10.0.1.1/24:Manager,Member']
        request['HTTP_X_FORWARDED_FOR'] = '10.0.1.1' # Sets IP in request
        self.assertEqual( helper.getRolesForPrincipal( None, request ), ['Member','Manager'])

        helper.ip_roles = ['10.0.1.1/24:Manager',
                           '10.0.1.1/16:Member']
        request['HTTP_X_FORWARDED_FOR'] = '10.0.255.0'
        self.assertEqual( helper.getRolesForPrincipal( None, request ), ['Member'])
        request['HTTP_X_FORWARDED_FOR'] = '10.0.1.1'
        self.assertEqual( helper.getRolesForPrincipal( None, request ), ['Member','Manager'])

    def test_extractCredentials( self ):
        helper = self._makeOne()
        response = FauxCookieResponse()
        request = FauxRequest(RESPONSE=response)

        helper.ip_roles = ['10.0.1.1/24:Manager,Member']
        request['HTTP_X_FORWARDED_FOR'] = '10.0.1.1' # Sets IP in request
        self.assertEqual( helper.extractCredentials( request ), {'AutoRole':True})

        request['HTTP_X_FORWARDED_FOR'] = '10.1.0.0' # Sets IP in request
        self.assertEqual( helper.extractCredentials( request ), {})

        request['HTTP_X_FORWARDED_FOR'] = '10.0.1.1' # Sets IP in request
        request._auth = "Basic" # No extraction if already http auth
        self.assertEqual( helper.extractCredentials( request ), {})

    def test_authenticateCredentials( self ):
        helper = self._makeOne()
        response = FauxCookieResponse()
        request = FauxRequest(RESPONSE=response)

        self.assertEqual( helper.authenticateCredentials( {} ), None)
        self.assertEqual( helper.authenticateCredentials( {'AutoRole':True} ), ('Anonymous User','Anonymous User'))
        # Make sure we don't instantiate anonymous if there are other credentials
        self.assertEqual( helper.authenticateCredentials( {'AutoRole':True, 'login':'user','password':'password'} ), None)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestAutoRole))
    return suite

if __name__ == '__main__':
    framework()
